import  click
import  pudb
from    pfmongo                 import  driver
from    argparse                import  Namespace
from    pfmongo                 import  env
from    pfmisc                  import  Colors as C
from    pfmongo.commands.clop   import add
from    pfmongo.config          import settings


NC  = C.NO_COLOUR
GR  = C.GREEN
CY  = C.CYAN

def delete_do(id:str, options:Namespace) -> int:
    thisCollection:str      = add.currentCollection_getName(options)
    delFail:int             = document_delete(id, options)
    if settings.appsettings.donotFlatten or delFail:
        return delFail
    options.collectionName  = add.shadowCollection_getName(options)
    delFail:int             = document_delete(id, options)
    options.collectionName  = thisCollection
    add.collection_connect(thisCollection, options)
    return delFail

def document_delete(id:str, options:Namespace) -> int:
    if env.env_failCheck(options):
        return 100
    options.do          = 'deleteDocument'
    options.argument    = id
    rem:int             = driver.run(options)
    return rem

@click.command(help=f"""
{C.CYAN}delete{NC} a document from a collection

This subcommand removes a document with passed 'id' from a collection.

The "location" is defined by the core parameters, 'useDB' and 'useCollection'
which are typically defined in the CLI, in the system environment, or in the
session state.

""")
@click.option('--id',
    type  = str,
    help  = \
    "Delete the document with the passed 'id'",
    default = '')
@click.pass_context
def delete(ctx:click.Context, id:str="") -> int:
    # pudb.set_trace()
    return delete_do(id, ctx.obj['options'])



import builtins
from pathlib            import Path

from _pytest.capture    import capsys
from pfmongo            import pfmongo
from pfmongo.__main__   import main
from pfmongo.pfmongo    import parser_setup, parser_JSONinterpret, parser_interpret
from argparse           import  ArgumentParser, Namespace
import  os
os.environ['XDG_CONFIG_HOME'] = '/tmp'
import              pudb
import              pytest
from                pytest_mock import mocker
from datetime       import datetime
import              time
from typing         import Any
from                unittest.mock   import patch

def CLIcall_parseForStringAndRet(capsys, cli:str, contains:str, exitCode:int) -> None:
    print(f'Testing {cli}')

    ret:int     = main(cli.split())
    captured    = capsys.readouterr()
    assert ret  == exitCode
    assert contains in captured.out

def test_main_manCore(capsys) -> None:
    """
    Test core man page of the app.
    """
    print(f'Testing man page')

    ret:int     = main(['--man'])
    captured    = capsys.readouterr()
    assert ret == 2
    assert '--useDB <DBname>' in captured.out

def test_main_version(capsys) -> None:
    """
    Test version of the app.
    """
    print(f'Testing version page')

    CLIcall_parseForStringAndRet(capsys, "--version", "Name", 1)

    #ret:int     = main(['--version'])
    #captured    = capsys.readouterr()
    #assert ret == 1
    #assert 'Name' in captured.out

def test_imp_help(capsys) -> None:
    with pytest.raises(builtins.SystemExit) as exit_info:
        CLIcall_parseForStringAndRet(capsys, "fs imp --help", "imports", 0)

    assert exit_info.value.args     == (0,)




import  click
import  pudb
from    pfmongo         import  driver
from    argparse        import  Namespace
from    pfmisc          import  Colors as C

NC  = C.NO_COLOUR
GR  = C.GREEN
CY  = C.CYAN

def collection_list(field:str, options:Namespace) -> int:
    options.do          = 'listDocument'
    options.argument    = field
    ls:int              = driver.run(options)
    return ls

@click.command(help=f"""
{C.CYAN}list{NC} all documents (read from the filesystem) in a collection

The "location" is defined by the core parameters, 'useDB' and 'useCollection'
which are typically defined in the CLI, in the system environment, or in the
session state.

""")
@click.option('--field',
    type        = str,
    help        = \
    "List the value of the named field",
    default     = '_id')
@click.pass_context
def list(ctx:click.Context, field:str) -> int:
    # pudb.set_trace()
    options:Namespace   = ctx.obj['options']
    return collection_list(field, options)



#!/usr/bin/env python3
#
# (c) 2023+ Fetal-Neonatal Neuroimaging & Developmental Science Center
#                   Boston Children's Hospital
#
#              http://childrenshospital.org/FNNDSC/
#                        dev@babyMRI.org
#

__version__ = "0.9.95"

import sys

from    pfmongo             import pfmongo
from    pfmongo             import env
from    pfmongo.commands    import fs
from    pfmongo.pfmongo     import parser_setup, parser_interpret, parser_JSONinterpret

try:
    from    .               import __pkg, __version__
except:
    from pfmongo            import __pkg, __version__

from    argparse            import ArgumentParser, Namespace
import  pudb
from    pfmisc              import Colors as C
from    typing              import Any, Literal
from    pfmongo.config      import settings
from    pfmongo.models      import dataModel
from    pathlib             import Path
import  appdirs
import  click
from    click.formatting    import wrap_text

from    pfmongo.commands    import database, collection, fs, man, state, document

NC  = C.NO_COLOUR
GR  = C.GREEN
CY  = C.CYAN

options:Namespace               = Namespace()

def sysargv_revamp(newarg:list[str]) ->list[str]:
    """
    Small but crucial method for "revamping" the sys.argv list that
    is passed to the main click function.

    The new list is returned in addition to being set in the sys
    module.

    :param newarg: a string list of "new" arguments
    """
    executable:str              = sys.argv[0]
    sys.argv                    = []
    sys.argv.append(executable)
    sys.argv.extend(newarg)
    return sys.argv

def main(argv:list[str]=[]) -> int:
    """
    The main entry point to the program. Can also be called from Python by
    passing along a "pseudo" argv list of strings (note the first element of
    the list MUST be the executable name).

    The "core" CLI args are processed here as well as some environmental checks
    are performed. If necessary, the application will "terminate" from this
    method before routing to the subcommands/groups.

    This function demonstrates a design pattern that mixes argparse and click
    parsing, hopefully leveraging something of the best of both worlds.

    :param argv: a CLI argv list of strings.
    """
    global options
    # pudb.set_trace()
    add_help:bool               = False
    parser:ArgumentParser       = parser_setup(
                                    'A client for interacting with mongo DBs',
                                    add_help
                                )

    # Should we show (and exit) some man page help?
    options, extra              = parser_interpret(parser, argv)

    if not env.env_statePathSet(options):
        return 10

    # This is the result of argparse
    if man.coreOptions_show(options):
        return man.man(options)

    # the click "app" interprets sys.argv, hence the revamping here
    newargv:list[str]           = sysargv_revamp(extra)
    return(app())

@click.group(help = pfmongo.package_description)
@click.option('--man',
              is_flag   = True,
              help      = 'show more detail about core OPTIONS')
@click.option('--version',
              is_flag   = True,
              help      = 'show program version')
@click.pass_context
def app(ctx:click.Context,
        man:bool, version:bool) -> int:
    """
    The main "app" -- it mostly serves as a point to route to the
    various subcommands and provide some in-line help.

    Note the --man and --version are added as options here only to
    be picked up by the "--help", but they are in fact handled by
    the parent main() method.

    :param ctx: a click "context" -- it is expanded here to transmit additional
                information to subcommands.
    :param man: a CLI --man parameter used to dispatch to the argparse handler
    :return: simply returns a 0:success or <N>:failure
    """

    global options
    ctx.obj                     = {}
    ctx.obj['options']          = options

    subcommand:str|None         = click.get_current_context().invoked_subcommand

    return 0

app.add_command(database.database)
app.add_command(collection.collection)
app.add_command(fs.fs)
app.add_command(state.state)
app.add_command(document.document)
#app.add_command(man.man)




from    pathlib     import  Path
from    argparse    import  Namespace
from    typing      import  Optional
from    pfmisc      import  Colors  as C
from    pfmongo.commands.docop import add, delete, search, showAll, get
import  click
import  pudb

NC  = C.NO_COLOUR
GR  = C.GREEN
CY  = C.CYAN

@click.group(help=f"""
             {GR}document{CY} <cmd>{NC} -- document commands

This command group provides mongo "document" level commands.

""")
def document():
    pass

document.add_command(add.add)
document.add_command(delete.delete)
document.add_command(search.search)
document.add_command(showAll.showAll)
document.add_command(get.get)



import  click
from    argparse    import  Namespace
import  pudb
from    typing      import  Optional, Union, Annotated
from    pfmisc      import  Colors as C
from    pfmongo     import  env, pfmongo, driver
from    pfmongo.models  import responseModel

NC = C.NO_COLOUR
GR = C.LIGHT_GREEN
CY = C.CYAN

def stateResponse_eval(mongodb:pfmongo.Pfmongo) -> pfmongo.Pfmongo:
    mongodb.responseData    = mongo_responseEmbed(
                                state_get(mongodb.args)
                              )
    mongodb.exitCode        = 0
    return mongodb

def state_get(options:Namespace) -> responseModel.stateResponse:
    state:responseModel.stateResponse   = responseModel.stateResponse()
    state.database          = env.DBname_get(options)
    state.collection        = env.collectionName_get(options)
    return state

def mongo_responseEmbed(state:responseModel.stateResponse) \
    -> responseModel.mongodbResponse:
    return pfmongo.responseData_build(
        {
            'status':   True,
            'connect':  state
        }, f"{state.database}/{state.collection}"
    )

@click.command(help=f"""
               {GR}showall{NC} -- show internal state values

This command shows internal program state. It accepts no arguments.

""")
@click.pass_context
def showAll(ctx:click.Context) -> int:
    # pudb.set_trace()
    return driver.run_intReturn(ctx.obj['options'], stateResponse_eval)



import  click
from    argparse    import  Namespace
from    pathlib     import  Path
from    pfmisc      import  Colors as C
from    pfmongo     import  driver

NC = C.NO_COLOUR
GR = C.LIGHT_GREEN
CY = C.CYAN

@click.command(help=f"""
               {GR}showall{NC} -- show all databases

This command shows all the databases available in a given mongodb
server. It accepts no arguments.

""")
@click.pass_context
def showAll(ctx:click.Context) -> None:
    # pudb.set_trace()
    options:Namespace   = ctx.obj['options']
    options.do          = 'showAllDB'
    showall:int         = driver.run(options)



from    argparse    import Namespace
import  click
from    pathlib     import Path
from    pfmisc      import Colors as C
from    pfmongo     import driver

NC = C.NO_COLOUR
GR = C.LIGHT_GREEN
CY = C.CYAN

def connectTo(database:str, options:Namespace) -> int:
    options.do          = 'connectDB'
    options.argument    = database
    return driver.run(options)

@click.command(help=f"""
               {GR}connect {CY}<database>{NC} -- connect to a mongo <database>

This command connects to a mongo database. A mongodb "server" can contain
several "databases".

""")
@click.argument('database',
                required = True)
@click.pass_context
def connect(ctx:click.Context, database:str) -> int:
    # pudb.set_trace()
    options:Namespace   = ctx.obj['options']
    return connectTo(database, options)



from    argparse    import Namespace
from    pathlib     import Path
from    pfmisc      import Colors as C
from    pfmongo.commands    import fs
import  click
import  pudb

NC  = C.NO_COLOUR
GR  = C.GREEN
CY  = C.CYAN

def cwd(options:Namespace):
   pass

@click.command(help=f"""
               {GR}cd {CY}[<path>]{NC} -- change directory

The 'cd' command "changes directory" within a mongodb between a
<database> level at the root `/` and a collection within a
<database>

""")
@click.argument('path',
                required = False)
@click.pass_context
def cd(ctx:click.Context, path:str) -> None:
    pudb.set_trace()
    options         = ctx.obj['options']
    target:Path     = Path('')
    if path:
        target = Path(path)
    sessionRoot:Path   = fs.root(options)





from    typing                      import Callable, Optional
from    argparse                    import Namespace
import  asyncio
from    asyncio                     import AbstractEventLoop
import  json
import  sys
from    pfmisc                      import Colors as C
from    pfmongo                     import pfmongo
from    pfmongo.models              import responseModel
from    pfmongo.pfmongo             import Pfmongo  as MONGO
from    pfmongo.config              import settings
import  pudb

from typing import Any, Dict, List, Union, cast
from pydantic import BaseModel

try:
    from    .               import __pkg, __version__
except:
    from pfmongo            import __pkg, __version__

NC  = C.NO_COLOUR
GR  = C.GREEN
CY  = C.CYAN

# Define a new type that includes all possibilities
NestedDict = Union[str, Dict[str, Any], List[Any]]

class SizeLimitedDict(BaseModel):
    value: NestedDict

def get_size(obj: NestedDict) -> int:
    size = sys.getsizeof(obj)
    if isinstance(obj, dict):
        size += sum([get_size(v) for v in obj.values()])
        size += sum([get_size(k) for k in obj.keys()])
    elif hasattr(obj, '__dict__'):
        size += get_size(obj.__dict__)
    elif hasattr(obj, '__iter__') and not isinstance(obj, (str, bytes, bytearray)):
        size += sum([get_size(i) for i in obj])
    return size

def size_limit(obj: Any, limit: int, depth: int) -> NestedDict:
    # if depth == 0 and sys.getsizeof(obj) > limit:
    size:int    = get_size(obj)
    if depth == 0 and size > limit:
        # return "size too large"
        return f">>>truncated<<<({str(size)} > {limit})"
    elif isinstance(obj, dict):
        return {k: size_limit(v, limit, depth - 1) for k, v in obj.items()}
    elif isinstance(obj, list):
        return [size_limit(elem, limit, depth - 1) for elem in obj]
    else:
        return obj

def model_pruneForDisplay(model:NestedDict) -> NestedDict:
    if not settings.appsettings.noResponseTruncSize:
        model = size_limit(model,
                           settings.mongosettings.responseTruncSize,
                           settings.mongosettings.responseTruncDepth)
    depthUp:int = 1
    while get_size(model) > settings.mongosettings.responseTruncOver:
        model = size_limit(model,
                           settings.mongosettings.responseTruncSize,
                           settings.mongosettings.responseTruncDepth-depthUp)
        depthUp += 1
    return model

def model_gets(mongodb:MONGO) -> Callable[[bool], str]:
    """ return the internal response model as a string """
    model:NestedDict            = {}
    modelForDisplay:NestedDict  = {}

    def model_toStr(addModelSizes:bool = False) -> str:
        respstr:str             = ""
        if not settings.appsettings.detailedOutput:
            return mongodb.responseData.message
        try:
            respstr = mongodb.responseData.model_dump_json()
        except Exception as e:
            respstr = '%s' % mongodb.responseData.model_dump()
        model           = json.loads(respstr)
        modelForDisplay = model_pruneForDisplay(model)
        respstr         = json.dumps(modelForDisplay)
        if addModelSizes:
            respstr    += json.dumps(
                            {
                                'modelSize': {
                                    'orig' : get_size(model),
                                    'disp' : get_size(modelForDisplay)
                                }
                            }
           )
        return respstr

    return model_toStr

def responseData_print(mongodb:MONGO) -> None:
    model_asString:Callable[[bool], str] = model_gets(mongodb)
    print(model_asString(settings.appsettings.modelSizesPrint))

def event_process(
    options:Namespace,
    f_syncCallBack:Optional[Callable[[MONGO], MONGO]] = None
):
    # Create the mongodb object...
    mongodb:pfmongo.Pfmongo     = pfmongo.Pfmongo(options)

    def payloadAs(returnType:str = 'int') -> int|pfmongo.Pfmongo:
        match returnType:
            case 'int':
                return mongodb.exitCode
            case 'obj':
                return mongodb
            case _ :
                return mongodb.exitCode

    def run(**kwargs) -> int|pfmongo.Pfmongo:
        nonlocal mongodb
        printResponse:bool      = False
        returnType:str          = 'int'
        for k,v in kwargs.items():
            if k == 'printResponse':    printResponse   = v
            if k == 'returnType':       returnType      = v

        if not f_syncCallBack:
            # run it asynchronously..!
            loop:AbstractEventLoop      = asyncio.get_event_loop()
            loop.run_until_complete(mongodb.service())
        else:
            # else run it with a synchronous callback
            mongodb     = f_syncCallBack(mongodb)
        if printResponse:
            responseData_print(mongodb)
        return payloadAs(returnType)

    return run

def do(
    options:Namespace,
    retType:str,
    f_syncCallBack:Optional[Callable[[MONGO], MONGO]] = None
) -> int | pfmongo.Pfmongo:
    f = event_process(options, f_syncCallBack)
    return f(printResponse = True, returnType = retType)

def run_intReturn(
    options:Namespace,
    f_syncCallBack:Optional[Callable[[MONGO], MONGO]] = None
) -> int:
    if not isinstance((result := do(options, 'int', f_syncCallBack)), int):
       raise TypeError("did not receive int as expected")
    return result

def run_modelReturn(
    options:Namespace,
    f_syncCallBack:Optional[Callable[[MONGO], MONGO]] = None
) -> pfmongo.Pfmongo:
    if not isinstance((result := do(options, 'model', f_syncCallBack)), pfmongo.Pfmongo):
       raise TypeError("did not receive model as expected")
    return result

def run(
    options:Namespace,
    f_syncCallBack:Optional[Callable[[MONGO], MONGO]] = None
) -> int:

    # Create the mongodb object...
    mongodb:pfmongo.Pfmongo     = pfmongo.Pfmongo(options)

    if not f_syncCallBack:
        # run it asynchronously..!
        loop:AbstractEventLoop      = asyncio.get_event_loop()
        loop.run_until_complete(mongodb.service())
    else:
        # else run it with a synchronous callback
        mongodb     = f_syncCallBack(mongodb)

    # print responses...
    responseData_print(mongodb)

    # and we're done.
    return mongodb.exitCode




import  click
import  pudb
from    pfmongo         import  driver
from    argparse        import  Namespace
from    pfmisc          import  Colors as C

from    pfmongo.commands.clop   import add

NC  = C.NO_COLOUR
GR  = C.GREEN
CY  = C.CYAN

def document_search(target:str, field:str, options:Namespace) ->int:
    thisCollection:str  = add.currentCollection_getName(options)
    flatCollection:str  = add.shadowCollection_getName(options)
    options.do          = 'searchDocument'
    options.argument    = {
            "field":        field,
            "searchFor":    target.split(','),
            "collection":   thisCollection
    }
    hits:int              = driver.run(options)
    add.currentCollection_getName(options)
    return hits

@click.command(help=f"""
{C.CYAN}search{NC} all documents in a collection for the union of tags in a
comma separated list.

The "hits" are returned referenced by the passed "field".
""")
@click.option('--target',
    type        = str,
    help        = \
    "A comma separated list. The logical OR of the search is returned",
    default     = '')
@click.option('--field',
    type        = str,
    help        = \
    "List the search hits referenced by this field",
    default     = '_id')
@click.pass_context
def search(ctx:click.Context, target:str, field:str) -> int:
    # pudb.set_trace()
    return document_search(target, field, ctx.obj['options'])



from    pathlib     import  Path
from    argparse    import  Namespace
from    typing      import  Optional
from    pfmisc      import  Colors  as C
import  click
import  pudb
from    pfmongo.commands.dbop   import connect, showAll, deleteDB


NC  = C.NO_COLOUR
GR  = C.GREEN
CY  = C.CYAN

@click.group(help=f"""
             {GR}database {CY}<cmd>{NC} -- database commands

This command group provides mongo "database" level commands.

""")
def database():
    pass

database.add_command(connect.connect)
database.add_command(showAll.showAll)
database.add_command(deleteDB.deleteDB)



import  click
import  pudb
from    pfmongo         import  driver
from    argparse        import  Namespace
from    pfmongo         import  env
import  json
from    typing          import  Union
from    pfmisc          import  Colors as C
from    pfmongo.commands.dbop   import  connect as db

NC  = C.NO_COLOUR
GR  = C.GREEN
CY  = C.CYAN

from pfmongo.models.dataModel import messageType

def DB_connectToTarget(DB:str, options:Namespace) -> str:
    currentDB:str  = env.DBname_get(options)
    if currentDB!= DB:
        db.connectTo(DB, options)
    return currentDB

@click.command(help=f"""
{GR}deletedb {C.CYAN}<database>{NC} delete an entire database

This subcommand removes an entire <database>.

""")
@click.argument('db',
                required = True)
@click.pass_context
def deleteDB(ctx:click.Context, db:str) -> int:
    # pudb.set_trace()
    options:Namespace   = ctx.obj['options']
    if env.env_failCheck(options):
        return 100
    DB_connectToTarget(db, options)
    options.do          = 'deleteDB'
    options.argument    = db
    rem:int             = driver.run(options)
    return rem



import  click
from    pathlib import Path
from    pfmisc  import Colors as C

NC  = C.NO_COLOUR
GR  = C.GREEN
CY  = C.CYAN

@click.command(help=f"""
{GR}imp {CY}<pathEX> <pathIN>{NC} -- import a file as a document

The {GR}imp{NC} command "imports" a file {CY}pathEX{NC} on the real filesystem
to the collection document in {CY}pathIN{NC}.


""")
@click.argument('pathEX',
                required = True)
@click.argument('pathIN',
                required = True)
def imp(path:str) -> None:
    # pudb.set_trace()
    target:Path     = Path('')
    if path:
        target = Path(path)





from    argparse    import Namespace
import  click
from    pathlib     import Path
from    pfmisc      import Colors as C
from    pfmongo     import driver, env

NC = C.NO_COLOUR
GR = C.LIGHT_GREEN
CY = C.CYAN

@click.command(cls=env.CustomCommand, help=f"""
Connect to a mongo <COLLECTION>

SYNOPSIS
{GR}connect {CY}<collection>{NC} -- connect to a mongo <collection>

DESC
This command connects to a mongo collection within a mongo database.
A mongodb "server" can contain several "databases", each of which
contains several "collections".

""")
@click.argument('collection',
                required = True)
@click.pass_context
def connect(ctx:click.Context, collection:str) -> int:
    # pudb.set_trace()
    options:Namespace   = ctx.obj['options']
    options.do          = 'connectCollection'
    options.argument    = collection
    return driver.run(options)



import  click
import  pudb
from    pfmongo         import  driver
from    argparse        import  Namespace
from    pfmongo         import  env
import  json
from    typing          import  Union
from    pfmisc          import  Colors as C
from    pfmongo.commands.clop         import  add

NC  = C.NO_COLOUR
GR  = C.GREEN
CY  = C.CYAN

from pfmongo.models.dataModel import messageType

def collection_delete(collection:str, options:Namespace) -> int:
    options.do          = 'deleteCollection'
    if env.env_failCheck(options):
        return 100
    collection_connectToTarget(collection, options)
    options.argument    = collection
    rem:int             = driver.run(options)
    return rem

def collection_connectToTarget(collection:str, options:Namespace) -> str:
    currentCol:str  = env.collectionName_get(options)
    if currentCol != collection:
        add.collection_connect(collection, options)
    return currentCol

@click.command(cls=env.CustomCommand, help=f"""
Deletes an entire <COLLECTION>

Use with care since there is no delete confirmation!

""")
@click.argument('collection',
                required = True)
@click.pass_context
def deleteCol(ctx:click.Context, collection:str) -> int:
    # pudb.set_trace()
    options:Namespace   = ctx.obj['options']
    return collection_delete(collection, options)



from    pathlib     import  Path
from    argparse    import  Namespace
from    typing      import  Optional
from    pfmisc      import  Colors  as C
import  click
import  pudb

NC  = C.NO_COLOUR
GR  = C.GREEN
CY  = C.CYAN

try:
    from    fop     import ls, cat, cd, mkdir, imp, exp
except:
    from    .fop    import ls, cat, cd, mkdir, imp, exp


def root(options:Namespace) -> Path:
    root:Path       = Path()
    RFS:Path        = options.thisSession
    splitPoint:str  = '_MONGO_'
    si:Optional[int]= next((i for i, segment in enumerate(RFS.parts) if splitPoint in segment), None)
    if si is not None:
        root        = Path(*list(RFS.parts)[:si + 1])
    return root

@click.group(help=f"""
             {GR}<fileSystem> {CY}cmd {NC}args -- "file system" commands

This command group uses file system (FS) "commands" in the context of a mongodb
allowing for an FS-modeled interface.

""")
def fs():
    pass

fs.add_command(ls.ls)
fs.add_command(cat.cat)
fs.add_command(cd.cd)
fs.add_command(mkdir.mkdir)
fs.add_command(imp.imp)
fs.add_command(exp.exp)




from    pathlib     import  Path
from    argparse    import  Namespace
from    typing      import  Optional
from    pfmisc      import  Colors  as C
import  click
import  pudb

NC  = C.NO_COLOUR
GR  = C.GREEN
CY  = C.CYAN

from    pfmongo.commands.stateop    import showAll

#try:
#    from    dbop    import  connect, showAll
#except:
#    from    .dbop   import  connect, showAll


@click.group(help=f"""
             {GR}state {CY}<cmd>{NC} -- internal state commands

This command group provides commands operating on internal state.

""")
def state():
    pass

state.add_command(showAll.showAll)




import  click
import  pudb
from    pfmongo         import  driver
from    argparse        import  Namespace
from    pfmongo         import  env
import  json
from    typing          import  Union
from    pfmisc          import  Colors as C

NC  = C.NO_COLOUR
GR  = C.GREEN
CY  = C.CYAN

from pfmongo.models.dataModel import messageType

def document_get(id:str, options:Namespace) -> int:
    options.do          = 'getDocument'
    if env.env_failCheck(options):
        return 100
    options.argument    = id
    read:int            = driver.run(options)
    return read

@click.command(help=f"""
{C.CYAN}get{NC} a document from a collection

This subcommand gets a document with passed 'id' from a collection.

The "location" is defined by the core parameters, 'useDB' and 'useCollection'
which are typically defined in the CLI, in the system environment, or in the
session state.

""")
@click.option('--id',
    type  = str,
    help  = \
    "If specified, get the document with the passed 'id'",
    default = '')
@click.pass_context
def get(ctx:click.Context, id:str="") -> int:
    # pudb.set_trace()
    return document_get(id, ctx.obj['options'])



import  click
from    pathlib import  Path
from    pfmisc  import  Colors as C

NC = C.NO_COLOUR
GR = C.LIGHT_GREEN
CY = C.CYAN

@click.command(help=f"""
               {GR}ls {CY}<args>{NC} -- list files

This command lists the objects (files and directories) that are at a given
path. This path can be a directory, in which case possibly multiple objects
are listed, or it can be a single file in which case information about that
single file is listed.

""")
@click.argument('path',
                required = False)
@click.option('--attribs',  required = False,
              help      = 'A comma separated list of file attributes to return/print')
@click.option('--long',
              is_flag   = True,
              help      = 'If set, use a long listing format')
def ls(path:str, attribs:str, long:bool) -> None:
    # pudb.set_trace()
    target:Path     = Path('')
    if path:
        target = Path(path)




import  click
from    pathlib import Path
from    pfmisc  import Colors as C

NC  = C.NO_COLOUR
GR  = C.GREEN
CY  = C.CYAN

@click.command(help=f"""
               {GR}mkdir {CY}<path>{NC} -- make directory

The 'mkdir' command "creates a new directory" within a mongodb.
Depending on the apparent "level" of the <path>, this is either
a new <database> off the root `/` or a new collection off the
root.

""")
@click.argument('path',
                required = False)
def mkdir(path:str) -> None:
    # pudb.set_trace()
    target:Path     = Path('')
    if path:
        target = Path(path)





import  click
from    pathlib import  Path
from    pfmisc  import  Colors as C

NC = C.NO_COLOUR
GR = C.LIGHT_GREEN
CY = C.CYAN

@click.command(help=f"""
               {GR}cat {CY}<file>{NC}  -- "Read" a <file>

This command shows the contents of <file>.

""")
@click.argument('path',
                required = False)
def cat(path:str) -> None:
    # pudb.set_trace()
    target:Path     = Path('')
    if path:
        target = Path(path)




import  click
import  pudb
from    pfmongo         import  driver
from    argparse        import  Namespace
from    pfmongo         import  env
import  json
from    typing          import  Union
from    pfmisc          import  Colors as C
from    pfmongo.config  import  settings

NC  = C.NO_COLOUR
GR  = C.GREEN
CY  = C.CYAN

from pfmongo.models.dataModel import messageType

def flatten_dict(data:dict, parent_key:str='', sep:str='/') -> dict:
    flattened:dict = {}
    for k, v in data.items():
        new_key = f"{parent_key}{sep}{k}" if parent_key else k
        if isinstance(v, dict):
            flattened.update(flatten_dict(v, new_key, sep=sep))
        elif isinstance(v, list):
            for i, item in enumerate(v):
                list_key = f"{new_key}{sep}{i}"
                if isinstance(item, (dict, list)):
                    flattened.update(flatten_dict({str(i): item}, parent_key=list_key, sep=sep))
                else:
                    flattened[list_key] = item
        else:
            flattened[new_key] = v
    return flattened

def env_OK(options:Namespace, d_doc:dict) -> bool|dict:
    envFailure:int    = env.env_failCheck(options)
    if envFailure: return False
    if not d_doc['status']:
        return not bool(env.complain(
            d_doc['data'], 1, messageType.ERROR
        ))
    if 'data' in d_doc:
        return d_doc['data']
    else:
        return False

def jsonFile_intoDictRead(filename:str) -> dict[bool,dict]:
    d_json:dict     = {
        'status':   False,
        'filename': filename,
        'data':     {}
    }
    try:
        f = open(filename)
        d_json['data']      = json.load(f)
        d_json['status']    = True
    except Exception as e:
        d_json['data']      = str(e)
    return d_json

def upload(d_data:dict, options:Namespace, id:str="") -> int:
    if id:
        d_data['_id']   = id
    d_data['_size']     = driver.get_size(d_data)
    options.do          = 'addDocument'
    options.argument    = d_data
    do:int              = driver.run(options)
    return do

def currentCollection_getName(options:Namespace) -> str:
    currentCol:str      = env.collectionName_get(options)
    if currentCol.endswith(settings.mongosettings.flattenSuffix):
        currentCol = currentCol.rstrip(settings.mongosettings.flattenSuffix)
        collection_connect(currentCol, options)
    return currentCol

def shadowCollection_getName(options:Namespace) -> str:
    sourceCol:str       = env.collectionName_get(options)
    shadowSuffix:str    = settings.mongosettings.flattenSuffix
    shadowCol:str       = sourceCol + shadowSuffix
    collection_connect(shadowCol, options)
    return shadowCol

def collection_connect(collection:str, options:Namespace) -> int:
    options.do          = 'connectCollection'
    options.argument    = collection
    return driver.run(options)

def add_do(document:dict, id:str, options:Namespace) -> int:
    thisCollection:str      = currentCollection_getName(options)
    # pudb.set_trace()
    saveFail:int            = upload(document, options, id)
    if settings.appsettings.donotFlatten or saveFail:
        return saveFail
    # pudb.set_trace()
    options.collectionName  = shadowCollection_getName(options)
    saveFail                = upload(flatten_dict(document), options, id)
    options.collectionName  = thisCollection
    connect:int             = collection_connect(thisCollection, options)
    return saveFail

def document_add(documentFile:str, id:str, options:Namespace) -> int:
    d_dataOK:dict|bool  = env_OK(options, jsonFile_intoDictRead(documentFile))
    d_data:dict         = {}
    if not d_dataOK:
        return 100
    if isinstance(d_dataOK, dict):
        d_data          = d_dataOK
    saveFail:int        = add_do(d_data, id, options)
    return saveFail

@click.command(help=f"""
{C.CYAN}add{NC} a document (read from the filesystem) to a collection

This subcommand accepts a document filename (assumed to contain JSON
formatted contents) and stores the contents in mongo.

A "shadow" document with a flat dataspace is also added to a "shadow"
collection. This "shadow" document facilitates searching and is kept
"in sync" with the orginal.

The "location" is defined by the core parameters, 'useDB' and 'useCollection'
which are typically defined in the CLI, in the system environment, or in the
session state.

""")
@click.option('--document',
    type  = str,
    help  = \
    "The name of a JSON formatted file to save to the collection in the database")
@click.option('--id',
    type  = str,
    help  = \
    "If specified, set the 'id' in the mongo collection to the passed string",
    default = '')
@click.pass_context
def add(ctx:click.Context, document:str, id:str="") -> int:
    # pudb.set_trace()
    return document_add(document, id, ctx.obj['options'])
